﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using System.Diagnostics;

namespace GraticuleVsMapUnit
{
    public partial class Form1 : Form
    {

        ManagedProj4Projection shapeProjection = new ManagedProj4Projection();
        ManagedProj4Projection graticuleProjection = new ManagedProj4Projection();
        GeographyUnit mapUnit = new GeographyUnit();

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            mapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.MapUnit = mapUnit;

            // Load countries on the screen
            ShapeFileFeatureLayer countriesLayer = new ShapeFileFeatureLayer(@"..\..\Data\Countries02.shp");
            countriesLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            countriesLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            shapeProjection.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            shapeProjection.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();

            // ############################################
            // 1.  Tell me why this code prevents the shape file from displaying.
            //     I don't understand why I can't assign the projection, but later I do this without problem.
            //shapeProjection.Open();
            //countriesLayer.FeatureSource.Projection = shapeProjection;
            // ############################################

            LayerOverlay countriesOverlay = new LayerOverlay();
            countriesOverlay.Layers.Add("countries", countriesLayer);
            winformsMap1.Overlays.Add("shapeOverlay", countriesOverlay);

            // Add a graticule adornment layer
            GraticuleAdornmentLayer graticuleAdornmentLayer = new GraticuleAdornmentLayer();
            graticuleProjection.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            
            // ############################################
            // 2.  Tell me why this commented line of code makes the graticule appear incorrect,
            //     but if I set the external projection to WGS-84 it looks right
            //graticuleProjection.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
            graticuleProjection.ExternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            // ############################################
            graticuleAdornmentLayer.Projection = graticuleProjection;
            graticuleAdornmentLayer.GraticuleLineStyle.OuterPen.Color = GeoColor.FromArgb(125, GeoColor.StandardColors.Blue);
            graticuleAdornmentLayer.GraticuleLineStyle.CenterPen.Color = GeoColor.FromArgb(125, GeoColor.StandardColors.Blue);
            winformsMap1.AdornmentOverlay.Layers.Add("graticule", graticuleAdornmentLayer);

            RectangleShape extents = new RectangleShape(-180, 90, 180, -90);
            winformsMap1.CurrentExtent = extents;

            winformsMap1.Refresh();
        }

        private void btnMapUnit_Click(object sender, EventArgs e)
        {

            RectangleShape extents = new RectangleShape(-180, 90, 180, -90);

            // Update the geography unit and projection
            if (mapUnit == GeographyUnit.DecimalDegree)
            {

                mapUnit = GeographyUnit.Meter;
                winformsMap1.MapUnit = mapUnit;
                ((Button)sender).Text = "Spherical Mercator";
                shapeProjection.Open();
                shapeProjection.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
                shapeProjection.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();

                winformsMap1.AdornmentOverlay.Layers.Clear();
                GraticuleAdornmentLayer graticuleAdornmentLayer = new GraticuleAdornmentLayer();
                graticuleAdornmentLayer.GraticuleLineStyle.OuterPen.Color = GeoColor.FromArgb(125, GeoColor.StandardColors.Blue);
                graticuleAdornmentLayer.GraticuleLineStyle.CenterPen.Color = GeoColor.FromArgb(125, GeoColor.StandardColors.Blue);
                graticuleAdornmentLayer.GraticuleTextFont = new GeoFont("Arial", 8, DrawingFontStyles.Bold);

                graticuleProjection.Open();
                // ############################################
                // 3.  Tell me why this code makes a graticule that shows up off west coast Africa with 
                //     Lat/Long coordinates even though we say external projection is Spherical Mercator.
                //     In fact, It seems I can put anything here and the result is the same.
                graticuleProjection.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
                graticuleProjection.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
                // ############################################
                graticuleAdornmentLayer.Projection = graticuleProjection;

                winformsMap1.AdornmentOverlay.Layers.Add("graticule", graticuleAdornmentLayer);

                extents = shapeProjection.ConvertToExternalProjection(extents);
                winformsMap1.CurrentExtent = extents;

                shapeProjection.Close();
            }
            else
            {
                mapUnit = GeographyUnit.DecimalDegree;
                winformsMap1.MapUnit = mapUnit;
                ((Button)sender).Text = "Decimal Degrees";
                shapeProjection.Open();
                shapeProjection.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
                shapeProjection.ExternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();

                winformsMap1.AdornmentOverlay.Layers.Clear();
                GraticuleAdornmentLayer graticuleAdornmentLayer = new GraticuleAdornmentLayer();
                graticuleAdornmentLayer.GraticuleLineStyle.OuterPen.Color = GeoColor.FromArgb(125, GeoColor.StandardColors.Blue);
                graticuleAdornmentLayer.GraticuleLineStyle.CenterPen.Color = GeoColor.FromArgb(125, GeoColor.StandardColors.Blue);
                graticuleAdornmentLayer.GraticuleTextFont = new GeoFont("Arial", 8, DrawingFontStyles.Bold);

                graticuleProjection.Open();
                graticuleProjection.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
                graticuleProjection.ExternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
                graticuleAdornmentLayer.Projection = graticuleProjection;
                winformsMap1.AdornmentOverlay.Layers.Add("graticule", graticuleAdornmentLayer);

                winformsMap1.CurrentExtent = extents;
            }

            //reproject the Overlays
            foreach (LayerOverlay overlay in winformsMap1.Overlays)
            {
                foreach (FeatureLayer layer in overlay.Layers)
                {
                    layer.Open();
                    layer.FeatureSource.Projection = shapeProjection;
                    layer.Close();
                }
            }

            winformsMap1.Refresh();

        }

        private void winformsMap1_MouseMove(object sender, MouseEventArgs e)
        {
            PointShape pointShape = ExtentHelper.ToWorldCoordinate(winformsMap1.CurrentExtent, new ScreenPointF(e.X, e.Y), winformsMap1.Width, winformsMap1.Height);

            if (mapUnit == GeographyUnit.DecimalDegree)
            {
                toolStripStatusLabel1.Text = "(Lat/Long) X:" + Math.Round(pointShape.X, 4) + " Y:" + Math.Round(pointShape.Y, 4);
            }
            else
            {
                toolStripStatusLabel1.Text = "(Spherical Mercator) X:" + Math.Round(pointShape.X, 4) + " Y:" + Math.Round(pointShape.Y, 4);
            }
        }


    }
}
